VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "MArray"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Description = "MArray is an array object that allows for extensible operations that arrays should be able to do trivially that VB can't do natively."
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
' All code Copyright (C) 1998 and Beyond Matthew Keller except
'  where explicitly stated otherwise.
' All code licensed under the GPL. See GPL.txt for details.
Option Explicit

Private mVarsize As Long ' Really the UBound. LBound is always 0
Private DataArray As Variant ' Where the data is

' The default props
Private Sub InitProperties()
    ' Set the Size on init
    mVarsize = 0
End Sub

' Allows an MArray to be initialized with data
Public Sub Initialize(initialValue As Variant, Optional commandDropFormatFlag As Boolean = False)
    ' the below flag is to handle the value given by the Command statement
    '  when someone has dragged a file(s)/folder(s) onto the program icon
    '  syntax is: "file\path\1" "file\path\elsewhere\2" "etc" ...
    If (commandDropFormatFlag) Then
        ' Call myself on the split of DblQuotaSpaceDblQuote
        Initialize Split(initialValue, Chr(34) & " " & Chr(34))
        
        'Iterate through DataArray and get rid of leading or trailing DblQuotes
        Dim c As Long
        c = 0
        While (c < mVarsize)
            If (Left(DataArray(c), 1) = Chr(34)) Then
                ' Leading, axe it
                DataArray(c) = Right(DataArray(c), Len(DataArray(c)) - 1)
            End If
            If (Right(DataArray(c), 1) = Chr(34)) Then
                ' trailing, axe it
                DataArray(c) = Left(DataArray(c), Len(DataArray(c)) - 1)
            End If
            c = c + 1
        Wend
    Else
        ' Set the DataArray to the initialValue
        DataArray = initialValue
        ' Set the size of the array (number of elements)
        mVarsize = UBound(DataArray) + 1
    End If
End Sub

' Return the contents of a slot. Does not change the array
Public Function See(ByVal Element As Long) As Variant
    If ((Element >= mVarsize) Or (Element < 0)) Then
        See = Empty
    Else
        See = DataArray(Element)
    End If
End Function

' Change the contents of a slot
Public Sub Change(ByVal Element As Long, ByVal datum As Variant)
    ' NOTE: We assume here that Element is a sane value. If not,
    '   Change silently returns and does nothing. Perhaps it should
    '   raise an error? In the mean time, code sanely.
    If ((Element < mVarsize) And (Element >= 0)) Then
        DataArray(Element) = datum
    End If
    
End Sub

' Unshift datum onto the array.
'   Unshifting is adding an item to the beginning of an array
Public Sub Unshift(ByVal datum As Variant)
    ' Resize the array, adding an empty first element
    Resize_Array "unshift"
    ' Fill the empty first element
    DataArray(0) = datum
End Sub

' Shift datum off of the array
'    Shifting is removing/returning the first item in an array
Public Function Shift() As Variant
    ' If we're being asked to shift off of an empty array, give them a null
    If (mVarsize = 0) Then
        Shift = Null
    Else
        Dim datum As Variant
        ' Trap the first element
        datum = DataArray(0)
        ' Resize the array, removing the first element
        Resize_Array "shift"
        Shift = datum
    End If

End Function

' Pop datum off of an array
'    Popping is removing/returning the last item in an array
Public Function Pop() As Variant
    ' If we're being asked to pop off of an empty array, give them a null
    If (mVarsize = 0) Then
        Pop = Null
    Else
        Dim datum As Variant
        ' Trap the last element
        datum = DataArray(UBound(DataArray))
        ' Resize the array, removing the last element
        Resize_Array "pop"
        Pop = datum
    End If

End Function

' Push datum onto the array
'   Pushing is adding an item to the end of an array
Public Sub Push(ByVal datum As Variant)
    ' Resize the array, adding an empty last element.
    Resize_Array "push"
    DataArray(UBound(DataArray)) = datum
End Sub

' How the public sees mvarSize
Public Property Get Size() As Long
    Size = mVarsize
End Property

'dump contents of DataArray into a message box -- not for the timid, should
'  be commented out in production builds
'Public Sub Dump()
'    Dim c As Long, msg As String, EOL As String
'    c = 0
'    EOL = Chr(13) & Chr(10)
'    If (IsEmpty(DataArray)) Then
'        msg = "0 = " & mVarsize & "?" & EOL
'    Else
'        msg = CStr(UBound(DataArray) + 1 & " = " & mVarsize & "?") & EOL
'        While c <= UBound(DataArray)
'            msg = msg & CStr(c) & ": " & DataArray(c) & EOL
'            c = c + 1
'        Wend
'    End If
'
'    MsgBox msg, vbInformation, "MArray Dump Daemon"
'
'End Sub


' Changes the dimensions of an array depending on the operation
'   That calls it
Private Sub Resize_Array(operation As String)
    Dim fromC As Long, toC As Long, newSize As Long, stopSpot As Long
    Dim newArray As Variant

    ' In order to keep this simple, I used keywords for the operations
    '   instead of a number. Perl is a great language, so I sponged off
    '   of it.
    Select Case LCase(operation)
        ' unshifting is adding an element to the beginning of an array
        Case "unshift"
            If (IsEmpty(DataArray)) Then
                newSize = 0
            Else
                newSize = UBound(DataArray) + 1
            End If
            stopSpot = newSize - 1
            toC = 1
            fromC = 0
    
        ' shifting is removing an element from the beginning of an array
        Case "shift"
            newSize = UBound(DataArray) - 1
            stopSpot = newSize + 1
            toC = 0
            fromC = 1
            
        ' pushing is adding an element to the end of an array
        Case "push"
            If (IsEmpty(DataArray)) Then
                newSize = 0
            Else
                newSize = UBound(DataArray) + 1
            End If
            stopSpot = newSize - 1
            fromC = 0
            toC = 0
            
        ' popping is removing an element from the end of an array
        Case "pop"
            newSize = UBound(DataArray) - 1
            stopSpot = newSize
            fromC = 0
            toC = 0
    End Select

    If (newSize < 0) Then
        ' the array is now empty
        mVarsize = 0
        newArray = Empty
    Else
        ' Make a new array of the size specified
        ReDim newArray(newSize)
        
        'Update the size
        mVarsize = newSize + 1

        ' Copy over the relevant elements from DataArray into newArray
        While (fromC <= stopSpot)
            newArray(toC) = DataArray(fromC)
            fromC = fromC + 1
            toC = toC + 1
        Wend
    End If

    ' assign it
    DataArray = newArray

End Sub

'reverses the elements in the array
Public Sub Reverse()

    Dim c As Integer, h As Integer, temp As Variant
    c = 0
    h = UBound(DataArray)
    
    ' Make a copy of DataArray
    temp = DataArray
    
    ' Copy DataArray's elements backwards into temp
    While (c <= h)
        temp(c) = DataArray(h - c)
        c = c + 1
    Wend
    
    ' Make DataArray look like temp
    DataArray = temp
    
End Sub

' return the variant that IS the array
Public Function Give() As Variant

    Give = DataArray
    
End Function
